module NotificationsApiCommonController
  extend ActiveSupport::Concern

  included do
    before_action :set_targetable, except: %i[move]

    private

    def set_targetable
      # Not dynamic to prevent attacks
      targetable_class = [Hermes::Applicant, Hermes::Lessor, Hermes::Entity, User].find do |targetable|
        targetable.name == params[:targetable_type]
      end

      @targetable = targetable_class.find params[:id]

      render_json_error(:not_found, :object_not_found) unless @targetable
    end


    def set_notifications
      limit = params[:limit] || params[:per_page]
      limit ||= 0

      return @notifications = @targetable.unopened_notifications.limit(limit) if action_name.to_sym == :open_all

      case params[:filter]&.to_sym
      when :all
        return @notifications = @targetable.notifications.limit(limit)
      when :opened
        return @notifications = @targetable.opened_notifications.limit(limit)
      when :unopened
        return @notifications = @targetable.unopened_notifications.limit(limit)
      else
        return @notifications = @targetable.notifications.limit(limit)
      end
    end

    def set_notification
      @notification = ::NotificationCenter::Notification.find params[:notification_id]

      render_json_error(:not_found, :object_not_found) unless @notification
    end


    def set_subscription
      @subscription = ::NotificationCenter::Subscription.find params[:subscription_id]

      render_json_error(:not_found, :object_not_found)  unless @subscription
    end

    def set_params
      @provider = params[:provider]&.to_sym || :email
      @key = params[:key]

      render_json_error(:not_found, :key_not_found) unless @key
    end

  end
end