# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::ApplicantsController < Hermes::Api::Dmo::ApplicationController
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s
  set_instance_for :eligible_requests

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ].freeze

  ACCEPTABLE_PARAMS = %w[
    ids
    nud
    temporary_id
    sne_profile_creation_date
    ss_id
    first_name
    last_name
    patronymic_name
    nationality
    email
    email_advances_assistant
    event_notification
    civility
    birth_date
    work_contract_type
    profession_details
    has_other_employers
    main_employer_name
    main_employer_siret
    workplace_postal_code
    workplace_locality
    foreign_workplace_postal_code
    foreign_workplace_locality
    foreign_workplace_country
    year_minus_one
    amount_year_minus_one
    year_minus_two
    amount_year_minus_two
    ressources_details
    tax_revenues
    apl_amount
    postal_details
    actual_housing_details
    family_situation
    family_situation_validated
    asked_children_nb
    valid_children_nb
    situation_tax_treshold
    status
    status_updated_at
    asked_locataire_hlm
    valid_locataire_hlm
    valid_priority_public
    asked_priority_public
    single_parent
    priority_public
    mobility
    mutation
    completed_requests_nb
    on_going_request_nb
    disability
    renewal_date
    eligibility_query
    active_housing_request_id
    housing_request_ids
    application_ids
    monthly_incomes
    monthly_charges
    cellphone
    sms_notification
    homephone
    workphone
    main_user_id
    address
    postal_code
    district
    country
    latitude
    longitude
    unemployment_period
    command_to_leave_date
    pro_or_home_tel_number
    unemployment_period_date_end
  ].freeze

  REGEX_PARAMS = %w[
  ].freeze

  MUTATE_PARAMS = %i[
    workplace_insee_code
    temporary_id
    nud
    sne_profile_creation_date
    ss_id
    first_name
    last_name
    patronymic_name
    nationality
    email
    email_advances_assistant
    event_notification
    civility
    birth_date
    work_contract_type
    profession_details
    has_other_employers
    main_employer_name
    main_employer_siret
    workplace_postal_code
    workplace_locality
    foreign_workplace_postal_code
    foreign_workplace_locality
    foreign_workplace_country
    year_minus_one
    amount_year_minus_one
    year_minus_two
    amount_year_minus_two
    ressources_details
    tax_revenues
    apl_amount
    postal_details
    actual_housing_details
    family_situation
    family_situation_validated
    situation_tax_treshold
    status
    asked_locataire_hlm
    valid_locataire_hlm
    valid_priority_public
    asked_priority_public
    priority_public
    mobility
    mutation
    disability
    renewal_date
    monthly_incomes
    monthly_charges
    cellphone
    is_owner
    sms_notification
    homephone
    workphone
    main_user_id
    address
    postal_code
    district
    country
    latitude
    longitude
    unemployment_period
    command_to_leave_date
    pro_or_home_tel_number
    unemployment_period_date_end
  ].freeze
        
  def mutate_params
    MUTATE_PARAMS
  end 

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS
    super
  end

  def user_is_allowed_to_remove_entity
    @instance.writable_by?(@user) || @instance.entity&.writable_by?(@user)
  end

  def all_eligibile_offers
    eligibility = JSON.parse(@instance.eligibility_query).each_with_object({}) do |(key, val), h|
      h[key.to_sym] = val
    end
    filter = build_query(Hermes::HousingOffer).deep_merge(eligibility)

    Hermes::HousingOffer.where(eligibility).where(build_query(Hermes::HousingOffer)).order(order_params)
  end

  def update
    return render_json_error(:not_found, :object_not_found) unless @instance
    return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && (@instance.writable_by?(@user) || @instance&.entity.writable_by?(@user))
    # return render_error_not_mutable(not_mutable) unless not_mutable.empty?

    param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
    mutate_valid_params[:user_id] = param_user if param_user && @user.has_role?('Administrator')
    return render_json_error(:parameters_not_present, :no_valid_parameters) unless mutate_valid_params.present?
    Rails.logger.debug "TEST valid_params are #{mutate_valid_params}"
    return respond_with_errors(@instance) unless @instance.update(mutate_valid_params) && @instance.save

    render(jsonapi: @instance, serializer: @serializer)
  end

  def eligible_offers
    return render_json_error(:not_found, :instance_not_found) unless @instance

    if params[:reserving_siren].present?
      rh = Hermes::ReservationHolder.find_by(siren: params[:reserving_siren])
      params[:reserving_siren] = { '$in' => rh.siren } if rh 
    end 

    acceptable_params *Hermes::Api::Dmo::HousingOffersController.acceptable_params
    regex_params *Hermes::Api::Dmo::HousingOffersController.regex_params
    valid_operators *VALID_OPERATORS
    options = {}

    paginate(
      all_eligibile_offers,
      each_serializer: Hermes::HousingOfferSerializer,
      **options
    )
  end

  def find_by_nud
    @instance = @model.find_or_create_by_nud(current_user, params[:nud], params[:birth_date])
    if @instance.errors.present?
      return render_json_error(:unprocessable_entity, :invalid_birth_date) if @instance.errors.messages[:birth_date]
      return render_json_error(:not_found, :invalid_nud) if @instance.errors.messages[:nud]
    end

    render(jsonapi: @instance, serializer: @serializer)
  end

  def switch
    @instance = Hermes::Applicant.find(params[:id]) if params[:id].present?
    return render_json_error(:not_found, :instance_not_found) unless @instance

    @target = User.find_by(main_applicant_id: @instance.id)
    @target = User.find_by(email: @instance.email) unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if current_user_roles.include?('entreprise')
      delegated_switch_response
    else
      render json: {
        success: false,
        message: 'Votre role doit être \'entreprise\' pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  private

  def user_can_destroy?
    (@instance.user_id.to_s == current_user.id.to_s) ||
      current_user.has_role?('Administrator') ||
      current_user.has_role?('Manager') ||
      current_user.has_role?('Superadmin')
  end

  def delegated_switch_response
    params.require(:entity_id)
    entity = Hermes::Entity.readable_by(@user).find_by(id: params[:entity_id])
    return render(json: { success: false, message: 'Entity not found' }, status: :not_found) unless entity
    return render(json: { success: false, message: 'No active housing_request for this applicant' }, status: :not_found) unless @instance.first_active_request


    search_params = {
      entity_id: entity.id, 
      housing_request_id: @instance.first_active_request&.id
    }
   
    entity_form = Hermes::HousingRequestEntityForm.find_by(search_params)
    return render(json: { success: false, message: 'delegation not found' }, status: :not_found) unless entity_form
    if entity_form.present? && entity_form.delegation && entity_form.delegation_signed
      success_switch_response
    else
      render json: {
        success: false,
        message: 'delegation not available or not signed'
      }, status: :unauthorized
    end
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end
