# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
module Hermes
  module Api
    module Dmo
      class LogsController < Hermes::Api::Dmo::ApplicationController
        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          ids
          created_at
          updated_at
          code
          read_level
          user_id
          action
          current_user_id
          applicant_id
          housing_offer_id
          housing_request_id
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        def initialize
          super
          @serializer = Hermes::LogSerializer
          @model = Log
        end

        def index
          paginate(index_aggregation, each_serializer: @serializer, **@options)
        end

        def housing_requests
          @query_precision ||= {}
          @query_precision = @query_precision.merge({ housing_request_id: { :$ne => nil } })
          index
        end

        def housing_offers
          @query_precision ||= {}
          @query_precision = @query_precision.merge({ housing_offer_id: { :$ne => nil } })
          index
        end

        def applicants
          @query_precision ||= {}
          @query_precision = @query_precision.merge({ applicant_id: { :$ne => nil } })
          index
        end

        def housing_applications
          @query_precision ||= {}
          @query_precision = @query_precision.merge({ housing_application_id: { :$ne => nil } })
          index
        end

        private

        def index_aggregation(target_model: @model, no_acl: false)
          return unless target_model

          precision = @query_precision || {}
          precision[:user_id] = params[:responsible] if params[:responsible]
          query = target_model
          query.where(build_query(target_model).deep_merge(precision)).order_by(order_params).includes(include_relationships)
        end
      end
    end
  end
end
