# frozen_string_literal: true

class AlsIdUniqJob < ActiveJob::Base
 
  queue_as :als_id_generation
  unique :until_executed

  def relaunch(id, force_als_id)
    Hermes::LogJob.add_to_queue(
      code: 'HOUSING_REQUEST_GENERATE_RELAUNCH',
      housing_request_id: id.to_s,
    )
    AlsIdUniqJob.set(wait: 1.minutes).perform_later(id, force_als_id)
  end 

  def perform(id, force_als_id = nil)
    Hermes::LogJob.add_to_queue(
      code: 'HOUSING_REQUEST_GENERATE_START',
      housing_request_id: id.to_s,
    )
    if hr = Hermes::HousingRequest.find(id)
      return unless hr.reload.als_id.blank? 
      not_unique = true
      max_try = 10
      counter = 0

      while not_unique && counter < max_try
        begin
          counter += 1
          break unless hr.reload.als_id.blank?
          generate_als_id(hr, force_als_id)
          hr.save!
          Hermes::LogJob.add_to_queue(
            code: 'HOUSING_REQUEST_GENERATE_SUCCESS',
            housing_request_id: id.to_s,
          )
          not_unique = false
        rescue => e
          if e.class == Mongo::Error::OperationFailure || /als_id_1 dup key/.match(e.message)
            sleep(0.1)
            retry if counter < max_try
            Hermes::LogJob.add_to_queue(
              code: 'HOUSING_REQUEST_GENERATE_ALS_ID_MAX_TRY',
              housing_request_id: id,
              data: e.message
            )
            relaunch(id, force_als_id)
          else
            Hermes::LogJob.add_to_queue(
              code: 'HOUSING_REQUEST_GENERATE_ALS_ID_UNKNOWED_ERROR',
              housing_request_id: id,
              data: e.message,
              level: 'error',
            )
            relaunch(id, force_als_id)
          end 
        end
      end
    end 
  end

  def generate_als_id(hr, force_als_id = nil)
    return hr.als_id = force_als_id unless force_als_id.nil?
    als_id = nil
    while als_id.nil? || Hermes::HousingRequest.find_by(als_id: als_id) 
      max = Hermes::HousingRequest.all.only(:als_id).sort({als_id:-1}).limit(1).first&.als_id || format('%010d', 0)
      als_id = format('%010d', max.to_i + 1)
      hr.als_id = als_id
      break
    end
  end
end