# frozen_string_literal: true

class ExecuteFlux36TaskJob < ActiveJob::Base
 
  queue_as :exsopralow
  unique :until_executed

  MAX_TRY = 5

  def perform(id, try_nb = 0)
    hr = Hermes::HousingRequest.find(id)
    if hr && hr.als_id && hr.entity_id 
      so = Hermes::BoClient::EntityRequestTask.new(housing_request_id: id)
      ExecuteSopraTaskJob.perform_later(so.endpoint, so.to_xml, "post", "Hermes::HousingRequest", id.to_s)
    else
      if try_nb >= MAX_TRY
        Hermes::LogJob.add_to_queue(
          level: 'error', 
          housing_request_id: id,
          code: "FLUX_36_ERROR_MISSING_ATTRIBUTES", 
          action: "flux 36 housing_request missing attributes", 
          data: {
            id: id,
            hr_found: !!hr,
            als_id: hr&.als_id,
            entity_id: hr&.entity_id
          }
        )
      else
        ExecuteFlux36TaskJob.set(wait: 1.minutes).perform_later(id.to_s, try_nb +1)
      end 
    end 
  end
end
