# frozen_string_literal: true

# Legende

module Hermes
  # ReservationHolder class
  class ReservationHolder
    include Hermes::Concerns::CommonFieldsConcern
    field :name, type: String
    field :siren, type: Array

    owner_field :user_id

    common_indexes *%i[
      name
      siren
      lessor_ids
      public_read
    ]

    has_and_belongs_to_many :lessors, class_name: 'Hermes::Lessor'

    before_save :before_save_actions

    validates_uniqueness_of :name

    def self.create_default
      Hermes::ReservationHolder.find_by(name: 'ALS') || Hermes::ReservationHolder.create(name: 'ALS', public_read: true, siren: ['824541148', '784336992']) 
    end 

    def self.find_entities_from_siren(siren)
      sirens = Hermes::ReservationHolder.find_by(siren: siren)&.siren
      return if sirens.nil?
      if root_ids = Hermes::Entity.where(:siren.in => sirens).pluck(:root_id).uniq
        entities = Hermes::Entity.where(:root_id.in => root_ids).pluck(:id)
      end 
    end 

    def before_save_actions
      update_readers
    end
    
    def update_readers
      return self.readers = [] if public_read
      readers = []

      lessors.each do |l|
        readers += ["g_#{l.group_id}", "g_#{l.supervisor_group_id}", "g_#{l.manager_group_id}"]
      end 
      self.readers = readers
    end 

  end

end
  