# frozen_string_literal: true

module Hermes
  # Applicant serializer
  class AdminSearchRightSerializer < Hermes::ApplicationSerializer

    attributes %i[
      right
      type
      title
      group_id
      entity_id
      lessor_id
      user_id
      user_email
      user_first_name
      user_last_name
      user_phone_number
      user_auth_roles
      user_gender
      entity_als_id
      entity_title
      entity_siret
      entity_siren
      entity_id_cristal
      lessor_als_id
      lessor_title
      lessor_siret
      lessor_siren
      lessor_id_cristal
    ]
    def added_to_object_cache_key
      object&.user&.updated_at.to_i
    end 

    def responsible
      nil
    end

    def user_email
      object.user_email || object.user&.email
    end

    def user_first_name
      object.user_first_name || object.user&.first_name
    end

    def user_last_name
      object.user_last_name || object.user&.last_name
    end

    def user_phone_number
      object.user_phone_number || object.user&.phone_number
    end

    def user_auth_roles
      object.user_auth_roles || object.user&.auth_roles
    end

    def user_gender 
      object.user_gender || object.user&.gender
    end 

    [
      :entity_als_id,
      :entity_title,
      :entity_siret,
      :entity_siren,
      :entity_id_cristal,
      :lessor_als_id,
      :lessor_title,
      :lessor_siret,
      :lessor_siren,
      :lessor_id_cristal,
    ].each do |att|
      define_method(:"#{att}") do 
        type = att.to_s.include?('entity') ? 'entity' : 'lessor'
        return if object["#{type}_id"].nil?
        e = type == 'entity' ? Hermes::Entity.find(object.entity_id) : Hermes::Lessor.find(object.lessor_id) 
        return unless e
        e[att.to_s.gsub("#{type}_", '')]
      end
    end

  end
end
