require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    name: "String",
    url: "String",
    info_message: "String",
    department_codes: "Array"
  }

  ACCEPTABLE_PARAMS = []
  VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $eq
          $regex
          $all
        ]

  describe Api::Dmo::NudProvidersController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    check_mutable_params(
      model: Hermes::NudProvider, 
      mutate_params: MUTATE_PARAMS, 
      factory: :nud_provider,
      user_roles: ['Administrator']
    )


    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}

    #TESTS BASIQUES D'API

    describe 'POST /nud_providers' do
      describe 'Create nud_provider with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      
      
    end

    describe 'GET /nud_providers' do
      describe 'GET /nud_providers' do
        let!(:nud_provider1) {create(:nud_provider)}
        let!(:nud_provider3) {create(:nud_provider)}
        let!(:nud_provider2) {create(:nud_provider)}

        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end
    end

    describe 'GET /nud_providers/:id' do
      describe 'verif it work' do
        let!(:nud_provider1) {create(:nud_provider)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: nud_provider1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
  
    end


    describe 'GET /nud_providers/:departement' do
      describe 'verif it work with 2digit' do
        let!(:nud_provider1) {create(:nud_provider, department_codes: ["33"])}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: "33"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work with 3digit' do
        let!(:nud_provider1) {create(:nud_provider, department_codes: ["976"])}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: "976"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif not found' do
        let!(:nud_provider1) {create(:nud_provider, department_codes: ["976"])}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: "00"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /nud_providers/:id' do

    end

    describe 'DELETE /nud_providers/:id' do
      let!(:nud_provider) {create(:nud_provider, user_id: responsible.id.to_s)}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy', {
            id: nud_provider.id.to_s
        }
      end
      it 'returns status code 204' do
        expect(response).to have_http_status(204)
      end
    end

  end

  # require_relative '../../swagger_helper'

  # describe Hermes::Api::Dmo::NudProvidersController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   TAGS_API_DMO_nud_providerS = 'API_DMO_nud_providerS'

  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes
  #   let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
  #   let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

  #   context 'as admin' do
  #     let!(:admin) {create(:administrator)}

  #     path '/api/dmo/nud_providers' do
  #       post 'Create nud_provider' do
  #         tags TAGS_API_DMO_nud_providerS
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         json_params = {}
  #         MUTATE_PARAMS.each do |key, val|
  #           json_params[key] = {type: nice_type(val)}
  #         end
  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)
  #         #jsonapi_params( **json_params )

  #         response '201', 'nud_provider created' do
  #           let(:Authorization) {"Bearer #{token_for(responsible)}"}
  #           let(:params) do
  #             jsonapi_attributes(
  #                {})
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'nud_providers'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/nud_providers' do
  #       get 'Retrieves all nud_providers' do
  #         tags TAGS_API_DMO_nud_providerS
  #         produces 'application/json'

  #         parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
  #         ACCEPTABLE_PARAMS.each do |acceptable_param|
  #           type = ''
  #           if MUTATE_PARAMS.include?(acceptable_param)
  #             type = nice_type(MUTATE_PARAMS[acceptable_param])
  #           end
  #           parameter name: acceptable_param.to_s, in: :query, type: type, required: false
  #         end
  #         # VALID_OPERATORS.each do |valid_operator|
  #         #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
  #         # end

  #         security [Bearer: {}]
  #         paginator

  #         response '200', 'nud_providers found' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let!(:nud_providers) {create_list(:nud_provider, 5)}

  #           include_context 'with integration test'
  #           include_context 'valid jsonapi collection' do
  #             let(:expected_collection) {nud_providers}
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/nud_providers/{id}' do
  #       get 'Retrieves a single nud_provider by id' do
  #         tags TAGS_API_DMO_nud_providerS
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '200', 'nud_provider found' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let(:nud_provider) {create(:nud_provider)}
  #           let(:id) {nud_provider.id}

  #           include_context 'with integration test'
  #           include_context 'valid jsonapi ressource' do
  #             let(:expected_ressource) {nud_provider}
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/nud_providers/{id}' do
  #       put 'edit nud_provider' do
  #         tags TAGS_API_DMO_nud_providerS
  #         produces 'application/json'
  #         consumes 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]
  #         json_params = {}
  #         MUTATE_PARAMS.each do |key, val|
  #           json_params[key] = {type: nice_type(val)}
  #         end
  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)
  #         #jsonapi_params( **json_params )

  #         response '200', 'nud_provider updated' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let(:nud_provider) {create(:nud_provider)}
  #           let(:id) {nud_provider.id}
  #           let(:params) do
  #             jsonapi_attributes(
  #                { department_codes: ['01','02']}
  #             )
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           after do |_example|
            
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'nud_providers'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/nud_providers/{id}' do
  #       delete 'Delete a single nud_provider by id' do
  #         tags TAGS_API_DMO_nud_providerS
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '204', 'nud_provider deleted' do
  #           let(:Authorization) {"Bearer #{token_for(admin)}"}
  #           let(:id) {create(:nud_provider, responsible: admin.id).id}
  #           run_test!
  #         end
  #       end
  #     end
  #   end
  # end
end