require 'rails_helper'
require [Hermes::Engine.root, %w[app lib notifications_center user]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')


RSpec.describe NotificationCenter::Subscription do
  let!(:subscriber) { create(:user) }
  let!(:subscription) { create(:notifications_subscription, keys: [{name: 'testing1.key', email: true, slack: true}, {name: 'testing2.key', email: true, slack: true}]) }
  subject { create(:notifications_subscription, keys: [], subscriber: subscriber) }

  it 'should subscribe to key' do
    k = 'new.key'
    date_string = "2000-01-01 00:00:00 UTC"
    expect(subject.keys.first[:name]).to eq('testing1.key')
    expect(subject.keys.first[:updated_at].to_s).not_to eq(date_string)
    
    Timecop.freeze(DateTime.new(2000, 1, 1, 0, 0, 0))
    subject.subscribe_to_email(k)
    Timecop.return
    
    expect(subject.keys.last[:name]).to eq(k)
    expect(subject.updated_at.utc.to_s).to eq(date_string)
    expect(subject.keys.last[:email]).to eq(true)
    expect(subject.keys.last[:updated_at].to_s).to eq(date_string)
    expect(subject.keys.first[:name]).to eq('testing1.key')
    expect(subject.keys.first[:updated_at].to_s).not_to eq(date_string)
    expect(Hermes::UpdateSubscription.where(object_id: subscriber.id).count).to eq(3)
    expect(Hermes::UpdateSubscription.find_by(object_id: subscriber.id, name: k).updated_at.utc.to_s).to eq(date_string)
    expect(Hermes::UpdateSubscription.find_by(object_id: subscriber.id, name: 'testing1.key').updated_at.utc.to_s).not_to eq(date_string)
    expect(Hermes::UpdateSubscription.find_by(object_id: subscriber.id, name: 'testing2.key').updated_at.utc.to_s).not_to eq(date_string)
  end

  it 'should unsubscribe to key' do
    k = 'testing1.key'
    date_string = "2001-01-01 00:00:00 UTC"

    expect(subject.keys.first[:name]).to eq('testing1.key')
    expect(subject.keys.first[:updated_at].to_s).not_to eq(date_string)
    expect(subject.keys.last[:name]).to eq('testing2.key')
    expect(subject.keys.last[:updated_at].to_s).not_to eq(date_string)

    Timecop.freeze(DateTime.new(2001, 1, 1, 0, 0, 0))
    subject.unsubscribe_to_email(k)
    Timecop.return

    expect(subject.keys.first[:name]).to eq(k)
    expect(subject.updated_at.utc.to_s).to eq(date_string)
    expect(subject.keys.first[:email]).to eq(false)
    expect(subject.keys.first[:updated_at].to_s).to eq(date_string)
    expect(subject.keys.last[:name]).to eq('testing2.key')
    expect(subject.keys.last[:updated_at].to_s).not_to eq(date_string)
    expect(Hermes::UpdateSubscription.find_by(object_id: subscriber.id, name: k).updated_at.utc.to_s).to eq(date_string)
    expect(Hermes::UpdateSubscription.find_by(object_id: subscriber.id, name: 'testing2.key').updated_at.utc.to_s).not_to eq(date_string)
  end
end
