# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

HERMES_CONFIG['identity_provider_key'] = 'als_hermes_salarie'

RSpec.describe Hermes::RexClient::UpdateContactTask, type: :helper do
  5.times do |i|
    let!(:"entity#{i+1}") { create(:entity, id_cristal: "#{i+1}", siret: "0000000000000#{i+1}") }
  end 

  let!(:user) { create(:user, email: 'deborah.meliani@yopmail.com', password: Devise.friendly_token[0,20], group_ids: [entity2.group_id]) }
  let!(:user_right1) { create(:user_right, user: user, type: 'entity', entity_id: entity5.id, right: 'administrator') }
  let!(:rex_entity_deborah) do 
    {
      "fields" =>  {
        "nom" =>  "MELIANI",
        "rolecrelocal" =>  "CRE",
        "id_cristal" =>  "1",
        "prenom" =>  "DEBORAH",
        "email" =>  "deborah.meliani@yopmail.com",
        "id_etablissement_cristal" =>  [
          "1",
          "3",
          "4",
        ],
      },
    }
  end 

  describe 'Existing user' do
    it 'update user' do
      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah)

      expect(user.reload.first_name).to eq "DEBORAH"
      expect(user.reload.auth_roles).to eq ["collaborateur offre logement", "cre"]
      expect(user.reload.id_cristal).to eq "1"
      expect(User.all().count).to eq(1)
    end

    it 'update access' do 
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).not_to be_nil  

      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah)

      user.reload
      expect(Hermes::Entity.writable_by(user).count).to eq(3)  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity1.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity2.id)).to be_nil
      expect(Hermes::Entity.writable_by(user).find_by(id: entity3.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity4.id)).not_to be_nil  
      expect(Hermes::Entity.writable_by(user).find_by(id: entity5.id)).to be_nil  
    end 

    it 'update cre_email' do 
      entity5.set(updated_at: DateTime.current, cre_email: user.email)

      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity_deborah)

      expect(entity1.reload.cre_email).to eq(user.email)
      expect(entity1.reload.cre_first_name).to eq("DEBORAH")
      expect(entity1.reload.cre_last_name).to eq("MELIANI")
      expect(entity2.reload.cre_email).to eq(nil)  
      expect(entity3.reload.cre_email).to eq(user.email)  
      expect(entity4.reload.cre_email).to eq(user.email)  
      expect(entity5.reload.cre_email).to eq(nil)  
    end 
  end

  describe 'Not existing user' do
    it 'create user' do
      rex_entity = {
        "fields" =>  {
          "nom" =>  "Doe",
          "id_etablissement_cristal" =>  [
            "5",
          ],
          "rolecrelocal" =>  "CRE",
          "id_cristal" =>  "2",
          "prenom" =>  "John",
          "email" =>  "john.doe@yopmail.com"
        },
      }
      Hermes::RexClient::UpdateContactTask.new.update_contact(rex_entity)

      created_user = User.find_by(email: 'john.doe@yopmail.com')
      expect(User.all().count).to eq(2)
      expect(created_user.first_name).to eq "John"
      expect(Hermes::Entity.writable_by(user).count).to eq(1)  
      expect(Hermes::Entity.writable_by(created_user).find_by(id: entity5.id)).not_to be_nil
    end
  end
end
